#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "friction_writer.hh"
#include "tobool.hh"

template <class ScalarVector, class Vector>
FrictionWriter<ScalarVector, Vector>::FrictionWriter(
    Vector const &vertexCoordinates,
    Dune::BitSetVector<1> const &_boundaryNodes, std::string prefix,
    typename BW::Projector projector)
    : BW(vertexCoordinates, _boundaryNodes, prefix, projector),
      coefficientWriter(prefix + "Coefficients", std::fstream::out),
      stateWriter(prefix + "Alpha", std::fstream::out) {}

template <class ScalarVector, class Vector>
FrictionWriter<ScalarVector, Vector>::~FrictionWriter() {
  stateWriter.close();
  coefficientWriter.close();
}

template <class ScalarVector, class Vector>
void FrictionWriter<ScalarVector, Vector>::writeOther(
    ScalarVector const &coefficient, ScalarVector const &alpha) {
  for (size_t i = 0; i < boundaryNodes.size(); ++i) {
    if (!toBool(boundaryNodes[i]))
      continue;

    coefficientWriter << coefficient[i] << " ";
    stateWriter << alpha[i] << " ";
  }

  stateWriter << std::endl;
  coefficientWriter << std::endl;
}

#include "friction_writer_tmpl.cc"
