#ifndef SRC_STATE_HH
#define SRC_STATE_HH

#include <memory>

#include <dune/common/bitsetvector.hh>

#include "enums.hh"
#include "state/stateupdater.hh"
#include "state/ageinglawstateupdater.hh"
#include "state/sliplawstateupdater.hh"

template <class ScalarVector, class Vector>
std::shared_ptr<StateUpdater<ScalarVector, Vector>> initStateUpdater(
    Config::stateModel model, ScalarVector const &alpha_initial,
    Dune::BitSetVector<1> const &frictionalNodes, double L, double V0) {
  switch (model) {
    case Config::AgeingLaw:
      return std::make_shared<AgeingLawStateUpdater<ScalarVector, Vector>>(
          alpha_initial, frictionalNodes, L, V0);
    case Config::SlipLaw:
      return std::make_shared<SlipLawStateUpdater<ScalarVector, Vector>>(
          alpha_initial, frictionalNodes, L, V0);
    default:
      assert(false);
  }
}
#endif
